package ipm.ipmworks.control;

import ipm.ipmworks.service.FileService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.data.mongodb.gridfs.GridFsResource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("file")
public class FileController {

    @Autowired
    FileService fileService;

    @GetMapping("/get/{id}")
    public ResponseEntity<InputStreamResource> getFileById(@PathVariable("id") String fileId) throws IOException {
        GridFsResource gridFSFile = fileService.getFileById(fileId);
        if (gridFSFile == null)
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        try {
            return ResponseEntity.ok()
                    .contentLength(gridFSFile.contentLength())
                    .contentType(MediaType.valueOf(gridFSFile.getContentType()))
                    .body(new InputStreamResource(gridFSFile.getInputStream()));
        } catch (IOException e) {
            e.printStackTrace();
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        }
    }

    @GetMapping("/help/list")
    public List<Map> getListOfHelpFiles(){
        return fileService.getFileDescription("help");
    }

    @PostMapping(value="/add")
    public ResponseEntity<Boolean> addFile(@RequestPart(name="file", required = false) MultipartFile file,
                                            @RequestPart(name="description", required = false) String description,
                                            @RequestPart(value = "type", required = false) String type) {
        try {
            if (!fileService.uploadFile(file,description,type))
                return new ResponseEntity<>(false, HttpStatus.INTERNAL_SERVER_ERROR);
            return new ResponseEntity<>(true, HttpStatus.OK);
        }
        catch (Exception e) {
            e.printStackTrace();
            return new ResponseEntity<>(false, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping(value="/delete/{id}")
    public ResponseEntity<Boolean> deleteFile(@PathVariable("id") String id) {
        try {
            if (!fileService.deleteFile(id))
                return new ResponseEntity<>(false, HttpStatus.INTERNAL_SERVER_ERROR);
            return new ResponseEntity<>(true, HttpStatus.OK);
        }
        catch (Exception e) {
            e.printStackTrace();
            return new ResponseEntity<>(false, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

}

