package ipm.ipmworks.control;

import com.mongodb.BasicDBObject;
import com.mongodb.client.gridfs.GridFSFindIterable;
import com.mongodb.client.gridfs.model.GridFSFile;
import ipm.ipmworks.model.*;
import ipm.ipmworks.repository.*;
import ipm.ipmworks.service.EmailService;
import ipm.ipmworks.service.FileService;
import net.sf.jasperreports.engine.*;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;
import net.sf.jasperreports.engine.export.JRPdfExporter;
import net.sf.jasperreports.export.SimpleExporterInput;
import net.sf.jasperreports.export.SimpleOutputStreamExporterOutput;
import net.sf.jasperreports.export.SimplePdfReportConfiguration;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.InputStreamResource;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.gridfs.GridFsResource;
import org.springframework.data.mongodb.gridfs.GridFsTemplate;
import org.springframework.http.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.List;
import java.util.function.Consumer;
import java.io.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping(value="/resource")
public class ResourceController {

    @Autowired
    ResourceRepository rr;

    @Autowired
    RegionRepository regionRepository;

    @Autowired
    MongoTemplate mongoTemplate;

    @Autowired
    GridFsTemplate gridFsTemplate;

    @Autowired
    FileService fileService;

    @Autowired
    EmailService emailService;

    @Autowired
    @Qualifier("ipmDecisionRest")
    RestTemplate ipmDecisionRest;

    @Autowired
    @Qualifier("organicKnowledgeRest")
    RestTemplate organicKnowledgeRest;

    @Autowired
    ResourceTypeRepository resourceTypeRepository;

    @Autowired
    ProjectRepository projectRepository;
    @Autowired
    LanguageRepository languageRepository;

   @Autowired
   PestRepository pestRepository;

   @Autowired
   CropRepository cropRepository;

    @Value("${organic_knowledge.search.bykeword}")
    private String searchByKeyword;

    @Value("${organic_knowledge.search.bytitle}")
    private String searchByTitle;

    @RequestMapping(value="/saveResource", method= RequestMethod.POST)
    public Resource saveResource(@RequestBody Resource r){
        r.setHide(false);
        Resource res = rr.save(r);
        return res;
    }



    @RequestMapping(value="/getAllResources", method=RequestMethod.GET)
    public Page<Resource> getResources(@RequestParam(required = false) final Integer pageNumber, @RequestParam(required = false) final Integer pageSize) {
        int pSize = 10;
        int pNum = 0;
        if (pageSize != null) {
            pSize = pageSize;
        }
        if (pageNumber != null) {
            pNum = pageNumber;
        }
        Pageable pageable = PageRequest.of(pNum, pSize);
        return rr.findByApprovedIsTrue(pageable);
    }

    private Resource createResourceFromOrganicKnowledge(LinkedHashMap map){
        String title = (String) map.get("title");
        if(title != null && !title.isEmpty()) {
            Resource r = new Resource();
            r.setExternal(true);
            Link link =  new Link();
            link.setName("orgprint");
            link.setUrl("https://orgprints.org/");
            r.getLinks().add(link);
            r.setCanEdit(false);
            r.setProject("Organic Eprints");
            if (title != null) {
                r.setResourceName(title);
            }
            r.setIdResource("eprint_" + (Integer) map.get("eprintid") );

            if ((String) map.get("type") != null) {
                ResourceType temp = new ResourceType();
                temp.setName((String) map.get("type"));
                r.setResourceType(temp);

            }
            if ((String) map.get("institution") != null) {
                r.setContactInstitution((String) map.get("institution"));
                if ((String) map.get("department") != null) {
                    String inst = r.getContactInstitution() + ", " + (String) map.get("department");
                    r.setContactInstitution(inst);
                }
            }
            if ((String) map.get("abstract") != null) {
                r.setDescription((String) map.get("abstract"));
            }
            if ((List) map.get("doclang") != null) {
                String lang = (String) ((List<?>) map.get("doclang")).get(0);
                Language l = languageRepository.findByCode(lang);
                r.setLanguage(l);
            }
            if ((List<String>) map.get("documents") != null) {
                List<LinkedHashMap> documents = (List<LinkedHashMap>) map.get("documents");
                for (LinkedHashMap doc : documents) {
                    Link link1 =  new Link();
                    link1.setName((String) doc.get("uri"));
                    link1.setUrl((String) doc.get("uri"));
                    r.getLinks().add(link1);
                }
            }

            if ((String) map.get("uri") != null) {
                r.setResourceOrigin((String) map.get("uri"));
            }
            if ((String) map.get("contact_email") != null) {
                r.setContactEmail((String) map.get("contact_email"));
            }
            return r;
        }else{
            return null;
        }
    }
    private Resource createResourceFromIPMDecisions(LinkedHashMap map, List<String> acceptLanguage){
        Resource r = new Resource();
        r.setExternal(true);
        Link link =  new Link();
        link.setName("https://platform.ipmdecisions.net/");
        link.setUrl("https://platform.ipmdecisions.net/");
        r.getLinks().add(link);

        r.setCanEdit(false);
        if((String) map.get("resourceName")!=null){
            r.setResourceName((String) map.get("resourceName"));
        }
        if((String) map.get("resourceId")!=null){
            r.setIdResource(((String) map.get("resourceId")).replaceFirst(";","_"));
        }
        if((String) map.get("idResource")!=null){
            r.setIdResource(((String) map.get("idResource")).replaceFirst(";","_"));
        }
        if((String) map.get("project")!=null){
            r.setProject((String) map.get("project"));
        }
        if((String) map.get("description")!=null){
            r.setDescription((String)map.get("description"));
        }
        if((String) map.get("resourceOrigin")!=null){
            r.setResourceOrigin((String)map.get("resourceOrigin"));
        }
        if((String) map.get("contactInstitution")!=null){
            r.setContactInstitution((String)map.get("contactInstitution"));
        }
        if((String) map.get("contactEmail")!=null){
            r.setContactEmail((String)map.get("contactEmail"));
        }
        if((String) map.get("contactPhone")!=null){
            r.setContactPhone((String)map.get("contactPhone"));
        }
        if((String) map.get("citation")!=null){
            r.setCitation((String)map.get("citation"));
        }
        if((String) map.get("contactAddress")!=null){
            r.setContactAddress((String) map.get("contactAddress"));
        }
        if((String) map.get("resourceType")!=null){
            ResourceType resourceType = resourceTypeRepository.findByName((String)map.get("resourceType"));
            r.setResourceType(resourceType);
        }
        if((List<String>) map.get("regions")!=null){
            List<String> regions =(List<String>) map.get("regions");
            for(String reg:regions) {
                Region region = regionRepository.findByCode(reg);
                r.getRegions().add(region);
            }
        }
        if((List<String>) map.get("languages")!=null){
            List<String> languages =(List<String>) map.get("languages");
           if (acceptLanguage != null) {
                if (Collections.disjoint(languages, acceptLanguage)) {
                    return null;
                }
            }

            if (languages.size() > 1){
                Language temp = new Language(String.join(", ",languages));
                r.setLanguage(temp);
            }else{
                for(String lan:languages) {
                    Language l = languageRepository.findByName(lan);
                    r.setLanguage(l);
                }
            }

        }
        if((List<String>) map.get("pests")!=null){
            List<String> pests =(List<String>) map.get("pests");
            for(String pest:pests) {
                List<Pest> pestList = pestRepository.findByEPPO(pest);
                r.getPests().addAll(pestList);
            }
        }
        if((List<String>) map.get("crops")!=null){
            List<String> crops =(List<String>) map.get("crops");
            for(String crop:crops) {
                List<Crop> croplist = cropRepository.findByEPPO(crop);
                r.getCrops().addAll(croplist);
            }
        }
        return r;

    }
    @GetMapping(value = {"/get/{idResource}","/get/{idResource}/{lang}"})
    public Resource getResource(@PathVariable("idResource") String idResource, @PathVariable(value = "lang", required = false) String lang){
        Optional<Resource> resource = rr.findById(idResource);
        if (resource.isEmpty()){
          if (idResource.startsWith("eprint")){
                String idRes = idResource.substring(7);
               /* byte[] titleBytes = Base64.getDecoder().decode(idRes.getBytes());
                String title = new String(titleBytes);
                List<LinkedHashMap> map = organicKnowledgeRest.getForObject(searchByTitle+title+"|", List.class);*/
                List<LinkedHashMap> map = organicKnowledgeRest.getForObject(searchByTitle+idRes, List.class);
                return createResourceFromOrganicKnowledge(map.get(0));
            }else{
                //ovo je ipm decision
                String idRes = idResource.replaceFirst("_",";");
                HttpHeaders headers = new HttpHeaders();
                if (lang != null){
                    headers.set("Accept-Language", lang);
                }else{
                    headers.set("Accept-Language", "en");
                }
                HttpEntity entity = new HttpEntity(headers);

                ResponseEntity<LinkedHashMap> map = ipmDecisionRest.exchange("/search/"+idRes, HttpMethod.GET, entity,LinkedHashMap.class);

                return createResourceFromIPMDecisions(map.getBody(),null);
            }
        }else{
            List <Map> files = getResourceFiles(idResource);
            Resource r = resource.get();
            r.setFiles(files);
            return r;
        }
    }
    @GetMapping("/imageExist/{type}/{idResource}")
    public boolean hasImage(@PathVariable("idResource") String idResource, @PathVariable("type") String type){
        Query query = new Query();
        query.addCriteria(Criteria.where("metadata.idResource").is(idResource).and("metadata.type").is(type));
        GridFSFile gridFSFile = gridFsTemplate.findOne(query);
        if (gridFSFile == null){
            return false;
        }else{
            return true;
        }
    }

    @GetMapping("/file/{name}")
    @ResponseBody
    public ResponseEntity<InputStreamResource> getResourceFile(@PathVariable("name") String name) {
        GridFsResource gridFSFile = gridFsTemplate.getResource(name);
        if (gridFSFile == null)
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        try {
            return ResponseEntity.ok()
                    .contentLength(gridFSFile.contentLength())
                    .contentType(MediaType.valueOf(gridFSFile.getContentType()))
                    .body(new InputStreamResource(gridFSFile.getInputStream()));
        } catch (IOException e) {
            e.printStackTrace();
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        }
    }
    @GetMapping("/image/raw/{idResource}/{type}")
    @ResponseBody
    public ResponseEntity<InputStreamResource> getImageRaw(@PathVariable("idResource") String idResource,@PathVariable("type") String type) {
        try {
            GridFsResource gridFSFile = fileService.getResourceImage(idResource,type);
            if (gridFSFile != null) {
                return ResponseEntity.ok()
                        .contentType(MediaType.parseMediaType(MediaType.IMAGE_JPEG_VALUE))
                        .contentType(MediaType.parseMediaType(MediaType.IMAGE_PNG_VALUE))
                        .contentType(MediaType.parseMediaType(MediaType.IMAGE_GIF_VALUE))
                        .body(new InputStreamResource(gridFSFile.getInputStream()));
            }else{
                return null;
            }
        }catch(IOException e){
                e.printStackTrace();
                return ResponseEntity.internalServerError().build();
        }

    }

    @GetMapping("/image/{idResource}")
    @ResponseBody
    public ResponseEntity<InputStreamResource> getImage(@PathVariable("idResource") String idResource) {
            Optional<Resource> res = rr.findById(idResource);
            String  imageName;
            Resource resource=null;
            if (res.isPresent()) {
                resource = res.get();
                if (resource.getProject()!=null && resource.getProject().equals("Smart Protect")){
                    imageName="smartprotect";
                }else if (resource.getProject()!=null && resource.getProject().equals("Endure")){
                    imageName="endure";
                }else if (resource.getSectors() != null && resource.getSectors().size() >1) {
                    imageName = "multi";
                }else if (resource.getSectors() != null && resource.getSectors().size() ==1) {
                    imageName = resource.getSectors().get(0).getSectorIcon();
                }
                else{
                    imageName = "multi";
                }
            }else if (idResource.startsWith("adas")) { //ovo je resurs iz IPM Decision
                imageName="ipmdecision";
            }else if (idResource.startsWith("eprint")){ //eprint baza
                imageName="organiceprint";
            }else{
                imageName="ipmdecision";
            }
                try {
                    InputStream logo = fileService.getResourceImage(imageName, "logo").getInputStream();

                    if(logo !=null){
                        return ResponseEntity.ok()
                                .contentType(MediaType.parseMediaType(MediaType.IMAGE_JPEG_VALUE))
                                .contentType(MediaType.parseMediaType(MediaType.IMAGE_PNG_VALUE))

                                .body(new InputStreamResource(logo));
                    }else{
                        return ResponseEntity.internalServerError().build();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                    return ResponseEntity.internalServerError().build();
                }
    }

    private InputStream createInputStream(Image generatedImg){
        try {
            ByteArrayOutputStream compressed = new ByteArrayOutputStream();
            ImageOutputStream outputStream = ImageIO.createImageOutputStream(compressed);
            ImageWriter jpgWriter = ImageIO.getImageWritersByFormatName("JPG").next();
            ImageWriteParam jpgWriteParam = jpgWriter.getDefaultWriteParam();
            jpgWriteParam.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
            jpgWriteParam.setCompressionQuality(0.6f);
            jpgWriter.setOutput(outputStream);
            jpgWriter.write(null, new IIOImage((RenderedImage) generatedImg, null, null), jpgWriteParam);
            jpgWriter.dispose();
            InputStream is = new ByteArrayInputStream(compressed.toByteArray());
            is.close();
            return is;
        }catch (Exception e){
            return null;
        }
    }
    private BufferedImage generateImage(String sectorImagePath,InputStream centralImage,String regionImagePath,InputStream projectLogo, String languageImagePath, String icon1, String icon2, String icon3, String icon4, String icon5) throws IOException {

        try {
            InputStream inputStream = this.getClass().getResourceAsStream("/jasper/ipmworks_template.jasper");
            File region = new ClassPathResource(regionImagePath).getFile();
            File language = new ClassPathResource(languageImagePath).getFile();
            File sector = new ClassPathResource(sectorImagePath).getFile();
            Image userDefinedImg = null;
            if(centralImage != null){
                userDefinedImg = ImageIO.read(centralImage);
            }
            Image sectorImage = ImageIO.read(sector);
            Image regionImage = ImageIO.read(region);
            Image projectLogoImg = null;
            Image icon1Img, icon2Img, icon3Img, icon4Img, icon5Img;
            File icon1File,icon2File,icon3File,icon4File,icon5File;
            Map templateMap = new HashMap<>();

            if (icon1 != null){
                icon1File =  new ClassPathResource(icon1).getFile();
                icon1Img = ImageIO.read(icon1File);
                templateMap.put("icon1", icon1Img);

            }
            if (icon2 != null){
                icon2File =  new ClassPathResource(icon2).getFile();
                icon2Img = ImageIO.read(icon2File);
                templateMap.put("icon2", icon2Img);

            }
            if (icon3 != null){
                icon3File =  new ClassPathResource(icon3).getFile();
                icon3Img = ImageIO.read(icon3File);
                templateMap.put("icon3", icon3Img);

            }
            if (icon4 != null){
                icon4File =  new ClassPathResource(icon4).getFile();
                icon4Img = ImageIO.read(icon4File);
                templateMap.put("icon4", icon4Img);

            }
            if (icon5 != null){
                icon5File =  new ClassPathResource(icon5).getFile();
                icon5Img = ImageIO.read(icon5File);
                templateMap.put("icon5", icon5Img);

            }
            if (projectLogo != null){
                projectLogoImg = ImageIO.read(projectLogo);
                templateMap.put("projectLogo", projectLogoImg);
            }
            Image languageImage = ImageIO.read(language);
            if(userDefinedImg != null){
                templateMap.put("userDefinedImg", userDefinedImg);
            }
            templateMap.put("sector", sectorImage);
            templateMap.put("region", regionImage);
            templateMap.put("language", languageImage);
            JasperPrint jasperPrint = JasperFillManager.fillReport(inputStream, templateMap, new JREmptyDataSource());
            inputStream.close();

            DefaultJasperReportsContext ctx = DefaultJasperReportsContext.getInstance();
            JasperPrintManager printManager = JasperPrintManager.getInstance(ctx);
            BufferedImage rendered_image = null;
            rendered_image = (BufferedImage) printManager.printPageToImage(jasperPrint, 0, 1f);
            return rendered_image;
        } catch (IOException | JRException ex) {
            ex.printStackTrace();
            return null;
        }

    }


    private List<Map> getResourceFiles(String idResource) {
        Query query = new Query();
        List<Map> files = new ArrayList<>();
        query.addCriteria(Criteria.where("metadata.idResource").is(idResource).and("metadata.type").is("resource"));
        GridFSFindIterable gridFSFiles = gridFsTemplate.find(query);
        gridFSFiles.forEach((Consumer<? super GridFSFile>) gridFSFile -> {
            Map m = new HashMap();
            m.put("description",gridFSFile.getFilename());
            m.put("fileIdentifier",gridFSFile.getObjectId().toHexString());
            files.add(m);
        });
        return files;
    }

    @PostMapping(value="/add/{idResource}")
    public ResponseEntity<Boolean> addFiles(@PathVariable("idResource") String idResource,@RequestPart(name="image", required = false) MultipartFile image,
                                            @RequestPart(name="logo", required = false) MultipartFile logo,
                                            @RequestPart(value = "files", required = false) MultipartFile[] files) {
        try {
            if (!uploadFile(idResource,image,logo,files))
                return new ResponseEntity<>(false, HttpStatus.INTERNAL_SERVER_ERROR);
            return new ResponseEntity<>(true, HttpStatus.OK);
        }
        catch (Exception e) {
            e.printStackTrace();
            return new ResponseEntity<>(false, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
    private boolean uploadFile(String idResource, MultipartFile image,MultipartFile logo,MultipartFile[] files) throws IOException {
        if (rr.findById(idResource).isEmpty()){
            return false;
        }else {

            if(image != null) {
                BasicDBObject metaDataImage = new BasicDBObject();
                metaDataImage.put("idResource", idResource);
                Query query = new Query();
                query.addCriteria(Criteria.where("metadata.idResource").is(idResource).and("metadata.type").is("image"));
                gridFsTemplate.delete(query);
                metaDataImage.put("type", "image");
                gridFsTemplate.store(image.getInputStream(), image.getOriginalFilename(), image.getContentType(), metaDataImage);
            }
            if(logo != null) {
                BasicDBObject metaDataLogo = new BasicDBObject();
                metaDataLogo.put("idResource", idResource);
                Query query = new Query();
                query.addCriteria(Criteria.where("metadata.idResource").is(idResource).and("metadata.type").is("logo"));
                gridFsTemplate.delete(query);
                metaDataLogo.put("type", "logo");
                gridFsTemplate.store(logo.getInputStream(), logo.getOriginalFilename(), logo.getContentType(), metaDataLogo);
            }
            if( files != null) {
                for (MultipartFile mf : files) {
                    BasicDBObject metaDataResource = new BasicDBObject();
                    metaDataResource.put("idResource", idResource);
                    metaDataResource.put("type", "resource");
                    gridFsTemplate.store(mf.getInputStream(), mf.getOriginalFilename(), mf.getContentType(), metaDataResource);
                }
            }
            return true;
        }
    }
   private String createSearchCriteria(SearchModel searchModel){
        List<String> criteria= new ArrayList<>();
        List<Sector> sectors = searchModel.getSector();
        List<ResourceType> resourceTypes = searchModel.getResourceType();
        List<Crop> crops = searchModel.getCrop();
        List<Pest> pests = searchModel.getPest();
        List<Language> languages = searchModel.getLanguage();
        List<Region> regions = searchModel.getRegion();
        Project project = searchModel.getProject();
        String title = searchModel.getTitle();

        if (title != null){
            criteria.add(title);
        }

        if(sectors !=null && !sectors.isEmpty()) {
            criteria.add(sectors.stream().map(Sector::getName).collect(Collectors.joining(",")));
        }
        if(resourceTypes !=null && !resourceTypes.isEmpty()) {
           criteria.add(resourceTypes.stream().map(ResourceType::getName).collect(Collectors.joining(",")));
        }
       if(crops !=null && !crops.isEmpty()) {
           criteria.add(crops.stream().map(Crop::getCommonName).collect(Collectors.joining(",")));
       }
       if(pests !=null && !pests.isEmpty()) {
           criteria.add(pests.stream().map(Pest::getCommonName).collect(Collectors.joining(",")));
       }
     /*  if(languages !=null && !languages.isEmpty()) {
           criteria.add(languages.stream().map(Language::getName).collect(Collectors.joining(",")));
       }*/
       if(regions !=null && !regions.isEmpty()) {
           criteria.add(regions.stream().map(Region::getName).collect(Collectors.joining(",")));
       }
       if(project !=null) {
           criteria.add(project.getName());
       }
        return String.join(",", criteria);

   }

    private List<Resource> searchOrganicKnowledge(SearchModel searchModel){
        List<Resource> result = new ArrayList();
        try{
            String searchCriteria = createSearchCriteria(searchModel);
            List<LinkedHashMap> hashMaps = organicKnowledgeRest.getForObject(searchByKeyword+searchCriteria, List.class);
            for (LinkedHashMap map : hashMaps) {
                Resource r = createResourceFromOrganicKnowledge(map);
                if(r !=null) {
                    result.add(r);
                }
            }
            return result;
        }catch (Exception e) {
            e.printStackTrace();
            System.out.println("Organic knowledge search error!");
            return result;
        }
    }
    private List<Resource> searchIPMDecision(SearchModel searchModel) throws Exception{
        List<Resource> resourceList = new ArrayList<>();
        String acceptLanguage = null;
        List<String> acceptedLang = null;
        IpmDecisionModel model = new IpmDecisionModel();
        if(searchModel.getPest()!= null && !searchModel.getPest().isEmpty()){
            for(Pest p: searchModel.getPest()) {
                model.getPests().add(p.getEPPO());
            }
        }
        if(searchModel.getCrop()!= null && !searchModel.getCrop().isEmpty()){
            for(Crop c: searchModel.getCrop()) {
                model.getCrops().add(c.getEPPO());
            }
        }
        if(searchModel.getLanguage()!=null && !searchModel.getLanguage().isEmpty()){
        List languages = new ArrayList();
        for(Language l: searchModel.getLanguage()){
            model.getLanguages().add(l.getCode());
            languages.add(l.getCode());
        }
        acceptLanguage = String.join(",",languages);
        acceptedLang = searchModel.getLanguage().stream().map(l -> l.getName()).collect(Collectors.toList());
        }
        if(searchModel.getRegion()!=null && !searchModel.getRegion().isEmpty()){
            for(Region r: searchModel.getRegion()){
                model.getRegions().add(r.getCode());
            }
        }
       if(!model.isEmpty()) {
           HttpHeaders headers = new HttpHeaders();
           headers.set("Accept-Language", acceptLanguage);
           model.getLanguages().clear();
           HttpEntity entity = new HttpEntity(model, headers);
           ResponseEntity<List> result = ipmDecisionRest.postForEntity("/search", entity, List.class);
           for (LinkedHashMap map : (List<LinkedHashMap>) result.getBody()) {
              Resource r = createResourceFromIPMDecisions(map,acceptedLang);
              if (r !=null) {
                  resourceList.add(r);
              }
           }
       }/*else{
          ResourceType type = resourceTypeRepository.findByName("IPM Decisions Model");
           if ((searchModel.getProject()!=null && searchModel.getProject().getName().equals("IPM Decisions")) || (searchModel.getResourceType() !=null && searchModel.isIPMDecision() )) {
               HttpHeaders headers = new HttpHeaders();
               headers.set("Accept-Language", acceptLanguage);
               HttpEntity entity = new HttpEntity(new LinkedHashMap<>(), headers);
               ResponseEntity<List> result = ipmDecisionRest.postForEntity("/search", entity, List.class);
               for (LinkedHashMap map : (List<LinkedHashMap>)result.getBody()) {
                   Resource r = createResourceFromIPMDecisions(map);
                   resourceList.add(r);
               }
           }
       }*/
        return resourceList;
    }

    @PostMapping("/search")
    public List<Resource> searchResource(@RequestBody  SearchModel searchModel) throws Exception {
        Query query = new Query();
        Criteria current = new Criteria();
        Criteria sectorsCriteria= null;
        Criteria regionsCriteria= null;
        Criteria resourceTypeCriteria= null;
        Criteria cropCriteria= null;
        Criteria pestCriteria= null;
        Criteria languageCriteria= null;
        Criteria titleCriteria = null;

        if(searchModel.getTitle() !=null){
            titleCriteria = new Criteria().where("resourceName").regex(".*"+searchModel.getTitle()+".*","i");
        }
        if(searchModel.getSector() !=null) {
            List<Criteria> sectors = new ArrayList<Criteria>();
            for (Sector s : searchModel.getSector()) {
                Criteria c = new Criteria().where("sectors.name").is(s.getName());
                sectors.add(c);
            }

            if (!sectors.isEmpty()) {
                sectorsCriteria = new Criteria();
                sectorsCriteria = sectorsCriteria.orOperator(sectors);
            }
        }
        if(searchModel.getRegion() !=null) {
            List<Criteria> regions = new ArrayList<Criteria>();
            for (Region r : searchModel.getRegion()) {
                if (r.getName().equalsIgnoreCase("All")){
                    Criteria c = new Criteria().where("regions.name").exists(true);
                    regions.add(c);
                }else {
                    Criteria c = new Criteria().where("regions.name").is(r.getName());
                    regions.add(c);
                }
            }
            if (!regions.isEmpty()) {
                regionsCriteria = new Criteria();
                regionsCriteria = regionsCriteria.orOperator(regions);
            }
        }
        if(searchModel.getResourceType() !=null) {
            List<Criteria> resourceTypes = new ArrayList<Criteria>();
            for (ResourceType r : searchModel.getResourceType()) {
                Criteria c = new Criteria().where("resourceType.name").is(r.getName());
                resourceTypes.add(c);
            }
            if (!resourceTypes.isEmpty()) {
                resourceTypeCriteria = new Criteria();
                resourceTypeCriteria = resourceTypeCriteria.orOperator(resourceTypes);
            }
        }
        if(searchModel.getCrop()!=null) {
            List<Criteria> crops = new ArrayList<Criteria>();
            for (Crop crop : searchModel.getCrop()) {
                if (crop.getCommonName().equalsIgnoreCase("All")){
                    Criteria c = new Criteria().where("crops.commonName").exists(true);
                    crops.add(c);
                }else {
                    Criteria c = new Criteria().where("crops.commonName").is(crop.getCommonName());
                    crops.add(c);
                }
            }
            if (!crops.isEmpty()) {
                cropCriteria = new Criteria();
                cropCriteria = cropCriteria.orOperator(crops);
            }
        }
        if(searchModel.getPest() !=null) {
            List<Criteria> pests = new ArrayList<Criteria>();
            for (Pest p : searchModel.getPest()) {
                if (p.getCommonName().equalsIgnoreCase("All")){
                    Criteria c = new Criteria().where("pests.commonName").exists(true);
                    pests.add(c);
                }else {
                    Criteria c = new Criteria().where("pests.commonName").is(p.getCommonName());
                    pests.add(c);
                }
            }
            if (!pests.isEmpty()) {
                pestCriteria = new Criteria();
                pestCriteria = pestCriteria.orOperator(pests);
            }
        }
        if(searchModel.getLanguage() !=null) {
            List<Criteria> languages = new ArrayList<Criteria>();
            for (Language l : searchModel.getLanguage()) {
                Criteria c = new Criteria().where("language.name").is(l.getName());
                languages.add(c);
            }
            if (!languages.isEmpty()) {
                languageCriteria = new Criteria();
                languageCriteria = languageCriteria.orOperator(languages);
            }
        }
        List<Criteria> andCriteria = new ArrayList<Criteria>();
        if (titleCriteria != null){
            andCriteria.add(titleCriteria);
        }
        if (sectorsCriteria != null){
            andCriteria.add(sectorsCriteria);
        }
        if (regionsCriteria != null){
            andCriteria.add(regionsCriteria);
        }
        if (resourceTypeCriteria != null){
            andCriteria.add(resourceTypeCriteria);
        }
        if (cropCriteria != null){
            andCriteria.add(cropCriteria);
        }
        if (pestCriteria != null){
            andCriteria.add(pestCriteria);
        }
        if (languageCriteria != null){
            andCriteria.add(languageCriteria);
        }
        if (searchModel.getProject()!= null){
            Criteria c = new Criteria().where("project").regex(".*"+searchModel.getProject().getName()+".*","i");
            andCriteria.add(c);
        }
        if(!andCriteria.isEmpty()){
            Criteria c = new Criteria().where("approved").is(true);
            andCriteria.add(c);
            Criteria c1 = new Criteria().where("hide").is(false);
            andCriteria.add(c1);
            current = current.andOperator(andCriteria);
            query.addCriteria(current);
            List<Resource> ipmWorksResult = mongoTemplate.find(query, Resource.class);
            try {
                List<Resource> ipmDecisionResult = searchIPMDecision(searchModel);
                ipmWorksResult.addAll(ipmDecisionResult);
            }catch (Exception e){
                e.printStackTrace();
                System.out.println("IpmDecision error!");
            }
            try {
                List<Resource> organicKnowledgeResult = searchOrganicKnowledge(searchModel);
                ipmWorksResult.addAll(organicKnowledgeResult);
            }catch (Exception e){
                System.out.println("organic eprints error!");
            }
            return ipmWorksResult;
        }else{
            return null;
        }

    }

   @PostMapping("download")
   public void downloadDetails(@RequestBody Resource resource, HttpServletResponse response){
       try {
           List<Resource> l = new ArrayList();
           l.add(resource);
           JRBeanCollectionDataSource dataSource = new JRBeanCollectionDataSource(l);
           InputStream inputStream = this.getClass().getResourceAsStream("/jasper/resurssheet.jasper");
           Map<String, Object> params = new HashMap<String, Object>();
           File logoFile =  new ClassPathResource("/images/icons/logo.jpg").getFile();
           Image logo = ImageIO.read(logoFile);
           params.put("logo", logo);
           JasperPrint jasperPrint = JasperFillManager.fillReport(inputStream, params, dataSource);
           inputStream.close();
           response.setContentType("application/pdf");
           response.addHeader("Content-Disposition", "inline; filename=resource.pdf;");
           OutputStream out = response.getOutputStream();

           JRPdfExporter exporter = new JRPdfExporter();
           exporter.setExporterInput(new SimpleExporterInput(jasperPrint));
           exporter.setExporterOutput(new SimpleOutputStreamExporterOutput(out));
           SimplePdfReportConfiguration configuration = new SimplePdfReportConfiguration();
           exporter.setConfiguration(configuration);
           exporter.exportReport();
       }catch (Exception e){
           e.printStackTrace();
       }
   }
    @PostMapping("comment/{idResource}")
    public void commentResource(@RequestBody Comment comment,@PathVariable("idResource") String idResource ){
        StringBuilder body = new StringBuilder();
        String resourceLink = "https://ipmworks.net/toolbox/en/#/resource/"+idResource;
        body.append("You have a new comment regarding resource " + resourceLink);
        body.append("\n");
        body.append("\n");
        body.append("-----------------------");
        body.append("\n");
        body.append(comment.getText());
        body.append("\n");
        body.append("-----------------------");
        body.append("\n");
        if (!comment.getCommenterEmail().isEmpty()) {
            body.append("You can reply on this comment using email left by commenter: " + comment.getCommenterEmail());
        }
        body.append("\n");
        body.append("\n");
        body.append("Best regards\n");
        body.append("IPMWorks toolbox team");
        emailService.sendEmail("IPMWORKS-WP4@inrae.fr","New resource comment",body.toString());
    }


}
