package ipm.ipmworks.security;

import ipm.ipmworks.model.AuthenticationRequest;
import ipm.ipmworks.model.AuthenticationResponse;
import ipm.ipmworks.model.User;
import ipm.ipmworks.repository.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.*;
import java.util.Optional;

@RestController()
@RequestMapping(value="/auth")

public class AuthenticationController {

    @Autowired
    AuthenticationManager authenticationManager;

    @Autowired
    private JavaMailSender javaMailSender;

    @Autowired
    IPMUserDetailsService userDetailsService;

    @Autowired
    UserRepository userRepository;

    @Autowired
    JWTUtil jwtUtil;

    @Autowired
    PasswordEncoder bCryptPasswordEncoder;

    @PostMapping(value = "/authenticate")
    public ResponseEntity<?> authenticate(@RequestBody AuthenticationRequest authenticationRequest) {
        try {
            authenticationManager.authenticate(new UsernamePasswordAuthenticationToken(
                    authenticationRequest.getUsername(), authenticationRequest.getPassword()));
        } catch (DisabledException e) {
            return new ResponseEntity<>("Disabled user", HttpStatus.BAD_REQUEST);
        } catch (BadCredentialsException e) {
            return new ResponseEntity<>("Incorrect username or password", HttpStatus.BAD_REQUEST);
        }  catch (Exception e){
            e.printStackTrace();
            return new ResponseEntity<>("Unknown error", HttpStatus.INTERNAL_SERVER_ERROR);
        }
        final IPMUserDetailsImpl userDetails = (IPMUserDetailsImpl)userDetailsService.loadUserByUsername(authenticationRequest.getUsername());
        final String token = jwtUtil.generateToken(userDetails);
        AuthenticationResponse authenticationResponse = new AuthenticationResponse();
        authenticationResponse.setToken(token);
        authenticationResponse.setName(userDetails.getName());
        authenticationResponse.setUsername(userDetails.getUsername());
        authenticationResponse.setRoles(userDetails.getRoles());

        return ResponseEntity.ok(authenticationResponse);
    }

    void sendEmail(String receiver, String subject, String body) {
        SimpleMailMessage msg = new SimpleMailMessage();
        msg.setTo(receiver);
        msg.setSubject(subject);
        msg.setText(body);
        javaMailSender.send(msg);
    }

    @PostMapping(value = "/startChangingPassword")
    public void startChangingPassword(@RequestBody String username) {
        final IPMUserDetailsImpl userDetails = (IPMUserDetailsImpl)userDetailsService.loadUserByUsername(username);
        if (userDetails != null){
            final String token = jwtUtil.generateToken(userDetails).substring(0,40);
            User user = userRepository.findByUsername(username).get();
            user.setRecoveryToken(token);
            userRepository.save(user);
            String link = "http://ipmworks.net/toolbox/#/changePassword/"+token;
            String body = "Someone requested change of your password. If that was not you please ignore this email. Otherwise, click on this link "+link+" (or copy this link to your browser) and follow instructions.";
            StringBuilder builder = new StringBuilder();
            builder.append("Dear "+userDetails.name+",\n");
            builder.append("\n");
            builder.append(body);
            builder.append("\n");
            builder.append("\n");
            builder.append("Best regards\n");
            builder.append("IPMWorks technical team");
             sendEmail(userDetails.username,"IPMWorks password change request",builder.toString());
        }
    }

    @GetMapping(value = "/validateToken/{token}")
    public ResponseEntity<?> findUser(@PathVariable("token") String token){
        Optional<User> userOpt = userRepository.findByRecoveryToken(token);
        if(userOpt.isPresent()){
            User user = userOpt.get();
            AuthenticationResponse authenticationResponse = new AuthenticationResponse();
            authenticationResponse.setUsername(user.getUsername());
            return ResponseEntity.ok(authenticationResponse);
        }else{
            return new ResponseEntity<>("Unknown error", HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PostMapping(value = "/changePassword")
    public ResponseEntity<?> changePassword(@RequestBody AuthenticationRequest authenticationRequest){
        Optional<User> userOpt= userRepository.findByUsername(authenticationRequest.getUsername());
        if(userOpt.isPresent()) {
            User user = userOpt.get();
            String newPassword = bCryptPasswordEncoder.encode(authenticationRequest.getPassword());
            user.setPassword(newPassword);
            user.setRecoveryToken("");
            userRepository.save(user);
            return ResponseEntity.status(HttpStatus.OK).build();
        }else{
            return new ResponseEntity<>("Unknown error", HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    }
